
# q-expansion of newform 1098.2.a.i, downloaded from the LMFDB on 03 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1098
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[1], [0], [-1], [-2], [-6], [0], [-3], [0], [1], [-6], [0], [3], [-12], [1], [12], [2], [0], [-1], [4], [13], [-9], [-14], [-3], [9], [-1], [-12], [-5], [15], [-8], [-4], [-15], [7], [4], [11], [6], [6], [-2], [-4], [-14], [-6], [21], [21], [-4], [12], [-26], [-7], [-12], [-2], [-14], [18], [-26], [-6], [25], [10], [28], [12], [-2], [-17], [-18], [-26], [24], [31], [-29], [0], [-2], [-21], [17], [-6], [7], [-11], [-6], [-25], [0], [-13], [32], [-24], [-2], [-35], [-27], [20], [-24], [-37], [-2], [16], [3], [-15], [-18], [2], [13], [-29], [20], [-22], [16], [32], [5], [32], [4], [-10], [1], [-21], [-44], [-26], [-40], [24], [36], [-28], [-18], [15], [-33], [13], [-21], [-16], [-7], [-10], [-38], [15], [-11], [3], [4], [13], [14], [-30], [-24], [-41], [46], [42], [-31], [48], [32], [22], [-3], [-1], [-11], [-22], [-18], [-24], [39], [-20], [-6], [46], [17], [2], [46], [17], [38], [-24], [42], [-46], [-28], [-28], [-1], [34], [-41], [-29], [23], [-40], [17], [-28], [47], [-44], [32], [17], [33], [19], [-24], [0], [-4], [23], [16], [52], [46], [38], [-33], [59], [52], [-24], [34], [18], [56], [1], [-16], [62], [50], [-48], [48], [38], [38], [44], [-10], [12], [32], [-12], [12], [-17], [-30], [4], [-45], [37], [22], [-28], [-9], [44], [2], [-38], [7], [-27], [48], [66], [27], [13], [67], [21], [37], [-68], [21], [-58], [-68], [64], [28], [-48], [-44], [-35], [-1], [-69], [22], [-35], [42], [-48], [-2], [33], [5], [-50], [22], [-1], [-44], [-75], [-10], [-4], [36], [-54], [24], [44], [-24], [14], [-57], [10], [-27], [-60], [-8], [-24], [-14], [-34], [-1], [-22], [39], [-60], [26], [28], [-66], [12], [-49], [49], [76], [-16], [-18], [28], [14], [30], [32], [9], [-2], [70], [-50], [-40], [38], [-57], [-84], [-54], [-60], [60], [-49], [-25], [-28], [31], [43], [54], [16], [-57], [-59], [-58], [26], [45], [-34], [-48], [-68], [-3], [43], [-66], [4], [9], [10], [10], [-13], [-32], [52], [-63], [44], [-26], [56], [37], [11], [49], [29], [-14], [-32], [90], [-28], [15], [-49], [75], [-2], [-32], [-12], [-44], [-27], [68], [-69], [29], [-1], [30], [92], [-37], [58], [27], [-4], [28], [22], [58], [-48], [-20], [74], [57], [6], [24], [-23], [-20], [65], [-65], [91], [-65], [23], [-8], [-2], [-73], [-42], [-66], [90], [46], [39], [-81], [-80], [38], [-12], [-84], [10], [-84], [-96], [74], [-11], [24], [20], [24], [-39], [32], [2], [-54], [-83], [74], [66], [44], [48], [34], [-46], [24], [24], [-1], [63], [-5], [44], [-76], [-26], [19], [-25], [-36], [90], [-28], [-78], [-18], [-94], [33], [-62], [7], [4], [-1], [-64], [47], [46], [-22], [-42], [20], [82], [58], [36], [66], [80], [-40], [-104], [25], [-14], [-70], [-105], [-73], [-90], [-36], [35], [-100], [60], [70], [15], [-24]]
